@@ -25,9 +25,11 @@ import tempfile
 from datetime import datetime
 import quip.field_builder as fb
 import quip.version_builder as vb
+import quip.external as external
 from argparse import RawTextHelpFormatter
 from colorama import Fore, Style, init
 from quip import __version__
+import keyring
 
 version = __version__
 UPDATE_ACTION = ["update", "u", "up"]
@@ -133,6 +135,10 @@ class Quip:
         parser_icon = subparsers.add_parser('clean', aliases=CLEAN_ACTION[1:], help='Clears the dist folders')
         parser_icon.add_argument('name', nargs="?", help='name of the project')
 
+        parser_icon = subparsers.add_parser('setup', help='Setup External Systems')
+        parser_icon.add_argument('--sonarqube', action='store_true', help='create SonarQube Project')
+        parser_icon.add_argument('name', nargs="?", help='name of the project')
+
         parser_icon = subparsers.add_parser('version', help='shows the version of the template/extension')
         parser_icon.add_argument('version_method', nargs="?", choices=["minor", "major", "release"], help='update the version of the project. Options: minor,major,release.')
         parser_icon.add_argument('--force', dest="forced_version", help='Force to change the version in all possible files')
@@ -157,17 +163,22 @@ class Quip:
             current_folder = os.getcwd()
             template_path = os.path.join(current_folder, "src", "templates", "template.json")
             if os.path.exists(template_path):
-                args.name = os.path.basename(current_folder)
-                # _json = self.read_template_json(template_path=template_path)
-                # template = (_json["templateType"] == "Script")
-                # if template:
-                #     args.name = "ut-" + _json["name"]
-                # else:
-                #     args.name = "ue-" + _json["name"]
-
+                # args.name = os.path.basename(current_folder)
+                _json = self.read_template_json(template_path=template_path)
+                template = (_json["templateType"] == "Script")
+                if template:
+                    args.name = "ut-" + _json["name"]
+                    args.template = True
+                else:
+                    args.name = "ue-" + _json["name"]
+                logging.info(f"Project name: {args.name}")
                 self.in_project_folder = True
             else:
                 logging.error("You are not in a project folder. Please specify the project name.")
+
+        if "template" in dir(args) and args.template == False and args.name.startswith("ut-"):
+            args.template = True
+
         return args
     
     def main(self):
@@ -195,6 +206,7 @@ class Quip:
                 self.update_rename_scripts()
             else:
                 self.update_project(self.args.uuid, self.args.new_uuid)
+            self.dump_fields(write=True)
         elif action in DELETE_ACTION:
             self.delete_project()
         elif action in CLONE_ACTION:
@@ -234,6 +246,8 @@ class Quip:
             if not self.uip_global_config.get("new", False):
                 QuipGlobalConfig().check_config(self.uip_global_config)
             sys.exit(0)
+        elif action == "setup":
+            self.create_external_systems()
         elif action in CLEAN_ACTION:
             self.clean_project()
             sys.exit(0)
@@ -271,10 +285,13 @@ class Quip:
         self._global_conf_defaults = self.uip_global_config.get("defaults", {})
         self._global_conf_extension = self.uip_global_config.get("extension.yml", {})
         self._global_conf_uip = self.uip_global_config.get("uip.yml", {})
+        self._global_conf_external = self.uip_global_config.get("external", {})
         self._version_files = self.uip_global_config.get("version_files", None)
         self.default_template = self._global_conf_defaults.get("template", "ue-task")
         self.project_name = self.format_ext_name(project_name)
+        self.extension_name = self.format_ext_name(project_name.lower())
         self.template_name = self.titleize(project_name)
+        self.use_keyring = self._global_conf_defaults.get("use_keyring")
         logging.debug(f"Project Name: {self.project_name}")
         logging.debug(f"Template Name: {self.template_name}")
 
@@ -296,14 +313,14 @@ class Quip:
         
         os.makedirs(self.project_name)
         self.uip_init(self.project_name, self.default_template)
-        self.update_extension_yaml(self.project_name)
+        self.update_extension_yaml(self.extension_name)
         self.update_uip_config(self.project_name)
         self.update_template_json(self.project_name)
 
     def update_project(self, update_uuid=False, update_new_uuid=False):
         logging.info(f"Updating extension {self.template_name}")
         if not self.args.template:
-            self.update_extension_yaml(self.project_name)
+            self.update_extension_yaml(self.extension_name)
             self.update_uip_config(self.project_name)
         else:
             self.update_script_config(self.project_name)
@@ -508,14 +525,11 @@ class Quip:
     def upload_template(self):
         zip_file_path = self.build_zip(self.project_name)
 
+        uac_url = self._global_conf_uip.get("url", "http://localhost:8080/uc")
         uac_user = self._global_conf_uip.get("userid", "ops.admin")
         # uac_pass = input(f"Enter password for {uac_user}: ")
-        uac_pass = getpass(prompt=f"Enter password for {uac_user}: ")
-        if len(uac_pass) == 0:
-            logging.debug("Using password from config")
-            uac_pass = self._global_conf_defaults.get("uac_password")
+        uac_pass = self.ask_password(uac_url, uac_user)
 
-        uac_url = self._global_conf_uip.get("url", "http://localhost:8080/uc")
         template_url = uac_url + "/resources/universaltemplate/importtemplate"
         logging.info(f"Uploading to controller ({uac_url})")
         with open(zip_file_path, "rb") as zipfile:
@@ -533,11 +547,7 @@ class Quip:
         uac_url = self._global_conf_uip.get("url", "http://localhost:8080/uc")
         template_url = uac_url + "/resources/universaltemplate"
         logging.info(f"Uploading to controller ({uac_url})")
-        uac_pass = input(f"Enter password for {uac_user}: ")
-        if len(uac_pass) == 0:
-            logging.debug("Using password from config")
-            uac_pass = self._global_conf_defaults.get("uac_password")
-            logging.debug(f"Password is {uac_pass}")
+        uac_pass = self.ask_password(uac_url, uac_user)
         payload = self.merge_template_scripts(self.project_name)
         logging.debug(f"Payload = {self.format_json(payload)}")
         
@@ -568,11 +578,7 @@ class Quip:
         uac_user = self._global_conf_uip.get("userid", "ops.admin")
         uac_url = self._global_conf_uip.get("url", "http://localhost:8080/uc")
         logging.info(f"Downloading template from controller ({uac_url})")
-        uac_pass = getpass(prompt=f"Enter password for {uac_user}: ")
-        if len(uac_pass) == 0:
-            logging.debug("Using password from config")
-            uac_pass = self._global_conf_defaults.get("uac_password")
-            logging.debug(f"Password is {uac_pass}")
+        uac_pass = self.ask_password(uac_url, uac_user)
 
         if template_name is None:
             template_name = self.template_name
@@ -617,11 +623,17 @@ class Quip:
         if os.path.exists(fields_path):
             with open(fields_path) as f:
                 conf = yaml.safe_load(f)
+                template_dict = fb.prepare_template_fields(conf)
                 new_fields = conf.get("fields", [])
+                
                 logging.debug("FIELDS: ", new_fields)
                 fields_dict = fb.prepare_fields(new_fields, code)
-                template_dict = fb.prepare_template_fields(conf)
                 template_dict["fields"] = fields_dict
+
+                new_events = conf.get("events", [])
+                logging.debug("EVENTS: ", new_events)
+                events_dict = fb.prepare_event_fields(new_events)
+                template_dict["events"] = events_dict
             
             logging.info("Updating template.json file")
             template = self.join_path("src", "templates", "template.json")
@@ -655,7 +667,11 @@ class Quip:
                 template_dict = fb.dump_template_fields(_json)
                 fields_dict = fb.dump_fields(_json.get("fields"))
                 template_dict["fields"] = fields_dict
+                events_dict = fb.dump_events(_json.get("events"))
+                template_dict["events"] = events_dict
                 yaml_dump = yaml.dump(template_dict, Dumper=MyDumper, default_flow_style=False, sort_keys=False, width=1000)
+                yaml_dump = yaml_dump.replace('fields:\n', '\nfields:')
+                yaml_dump = yaml_dump.replace('events:\n', '\nevents:')
                 if not write:
                     print(yaml_dump)
                 else:
@@ -666,6 +682,32 @@ class Quip:
             logging.error(f"ERROR: template.json file is missing! Path= {template}")
             sys.exit(1)
 
+    def create_external_systems(self):
+        sq_config = self._global_conf_external.get("sonarqube", {})
+        sq_url = sq_config.get("url", None)
+        if sq_url is None:
+            logging.warn("Sonarqube configuration is missing in the uip_config file.")
+            return False
+
+        use_token = sq_config.get("use_token", True)
+        if use_token:
+            sq_pass = self.ask_password(sq_url, "token", prompt="Please enter Personal Access Key: ")
+            username = sq_pass
+            sq_pass = ""
+        else:
+            username = sq_config.get("username", None)
+            if username is None:
+                logging.error("Sonarqube username configuration is missing in the uip_config file.")
+                return False
+
+            sq_pass = self.ask_password(sq_url, username, prompt=f"Please enter SonarQube password for {username}: ")
+        
+        ssl_verify = sq_config.get("ssl_verify", True)
+        sq = external.SonarQube(sq_url, (username, sq_pass), ssl_verify)
+        prefix = sq_config.get("prefix", "cs-uac")
+        sq_project_name = prefix + "-" + self.extension_name
+        sq.create_project(sq_project_name)
+
     def build_zip(self, project_name):
         with tempfile.TemporaryDirectory() as tmpdirname:
             logging.debug(f"Created temporary directory {tmpdirname}")
@@ -733,7 +775,7 @@ class Quip:
     def titleize(self, name):
         if name[:3] in ["ue-", "ut-"]:
             name = name[3:]
-        elif name.lower() != name:
+        if name.lower() != name:
             # if name has uppercase than use it as-is
             return name
         name = name.replace("_", " ")
@@ -742,6 +784,8 @@ class Quip:
         return name.title()
 
     def format_ext_name(self, name):
+        if name.lower() != name:
+            return name
         name = name.replace("_", "-")
         name = name.replace(" ", "-")
         name = name.replace("/", "")
@@ -766,22 +810,21 @@ class Quip:
     
     def run_uip(self, action):
         need_pass = False
+        uac_url = self._global_conf_uip["url"]
+        uac_user = self._global_conf_uip["userid"]
         if action in ["push_all", "push"]:
             additional_params = "-a" if action == "push_all" else ""
-            command = f'''uip push {additional_params} -i {self._global_conf_uip["url"]} -u {self._global_conf_uip["userid"]}'''
+            command = f'''uip push {additional_params} -i {uac_url} -u {uac_user}'''
             need_pass = True
         elif action == "pull":
             need_pass = True
-            command = f'''uip pull -i {self._global_conf_uip["url"]} -u {self._global_conf_uip["userid"]}'''
+            command = f'''uip pull -i {uac_url} -u {uac_user}'''
         elif action == "build":
             need_pass = False
             command = f'''uip build -a'''
 
         if need_pass:
-            uac_pass = getpass(prompt=f'''Enter password for {self._global_conf_uip["userid"]}: ''')
-            if len(uac_pass) == 0:
-                logging.debug("Using password from config")
-                uac_pass = self._global_conf_defaults.get("uac_password")
+            uac_pass = self.ask_password(uac_url, uac_user)
             os.environ["UIP_PASSWORD"] = uac_pass
         
         logging.info(f"Initializing the extension with command {command}")
@@ -885,7 +928,7 @@ class Quip:
             with open(template, "w") as f:
                 _json = json.loads(template_content)
                 if "extension" in _json and _json["extension"] is not None:
-                    _json["extension"] = project_name
+                    _json["extension"] = self.extension_name
                 _json["name"] = self.template_name
                 f.write(self.format_json(_json))
                 logging.debug("template.json file is updated")
@@ -1043,6 +1086,26 @@ class Quip:
     
     def get_new_uuid(self):
         return str(uuid.uuid4()).replace("-","")
+    
+    def ask_password(self, server, user_name, prompt=None):
+        if prompt is None:
+            prompt = f'''Enter password for {user_name}: '''
+        password = getpass(prompt=prompt)
+        if len(password) == 0:
+            if self.use_keyring:
+                logging.debug("Using password from Keyring")
+                # get password 
+                password= keyring.get_password(server, user_name)
+            else:
+                logging.error("Password is missing.")
+                return None
+        else:
+            if self.use_keyring:
+                logging.debug("Updating the password in Keyring")
+                # set password 
+                keyring.set_password(server, user_name, password)
+        
+        return password
 
 class QuipGlobalConfig:
     def __init__(self, config_file=None) -> None:
